import 'dart:io';
import 'package:jinja/jinja.dart';
import 'package:args/args.dart';

const stateless = 'stateless';

void main(List<String> arguments) {
  exitCode = 0; // presume success
  final parser = ArgParser()
    ..addFlag(stateless, negatable: false, abbr: 's');

  ArgResults argResults = parser.parse(arguments);
  final className = argResults.rest;

  generateWidget(className[0], stateful: argResults[stateless] as bool);
}

Future<void> generateWidget(String className, { bool stateful = true }) async {
  var renderer = TemplateRenderer();

  try {
    String path = 'lib/src/custom_components/$className';
    bool directoryExists = await Directory(path).exists();

    if (directoryExists == true) {
      throw('Directory $className exists already');
    } else {
      Directory(path).create();
      var viewModelTemplate = renderer.render('cli/templates/stateful_view_model.dart.jinja', {
        "className": className
      });
      
      if (stateful == true) {
        viewModelTemplate = renderer.render('cli/templates/stateless_view_model.dart.jinja', {
            "className": className
        });
      }

      final viewModel = File('$path/view_model.dart',);
      viewModel.writeAsString(viewModelTemplate);
      
      var viewTemplate = renderer.render('cli/templates/view_template.dart.jinja', {
        "className": className
      });
      final view = File('$path/view.dart',);
      view.writeAsString(viewTemplate);
      
      var mobileViewTemplate = renderer.render('cli/templates/mobile_view_template.dart.jinja', {
        "className": className
      });
      final mobileView = File('$path/mobile_view.dart',);
      mobileView.writeAsString(mobileViewTemplate);
    }
  } catch (e) {
    handleError(e.toString());
  }
}

class TemplateRenderer {
  Environment? renderer;

  TemplateRenderer() {
      renderer = Environment();
  }

  String render(String path, Map<String, dynamic>? params) {
    var file = File(path);

    if (false == file.existsSync()) {
        handleError('Cannot find template $path');
    }

    var templateContent = File(path).readAsStringSync();

    var template = renderer?.fromString(templateContent);

    return template!.render(params);
  }
}

Future<void> handleError(String errorMsg) async {
  stderr.writeln(errorMsg);
  exitCode = 2;
}