
import 'package:flutter/material.dart';
import 'package:nirrti/nirrti.dart';
import 'view_model.dart';

mixin DesktopViewMixin {
  render(BuildContext context, OwlnextActionsButtonState state) {
    return LayoutBuilder(
      builder: (BuildContext context, BoxConstraints constraints) {
        return SizedBox(
          width: constraints.maxWidth,
          child: CompositedTransformTarget(
            link: state.link,
            child: OverlayPortal(
              controller: state.portalController,
              child: SizedBox(
                width: constraints.maxWidth,
                child: Stack(
                  children: [
                    OwlnextButton(
                      callback: () {
                        state.portalController.toggle();
                      },
                      child: Text(state.widget.text),
                    ),
                    Positioned(
                      top: 8,
                      right: 8,
                      child: Icon(
                        Icons.arrow_drop_down,
                        color: state.widget.textColor
                      ),
                    )
                  ],
                )
              ),
              overlayChildBuilder: (BuildContext context) {
                return CompositedTransformFollower(
                  showWhenUnlinked: false,
                  link: state.link,
                  offset: const Offset(0, 0),
                  targetAnchor: Alignment.bottomLeft,
                  followerAnchor: Alignment.topLeft,
                  child: Align(
                    alignment: Alignment.topLeft,
                    child: Column(
                      children: [
                        ConstrainedBox(
                          constraints: BoxConstraints(
                            minWidth: constraints.maxWidth,
                            maxWidth: constraints.maxWidth,
                            maxHeight: 250,
                          ),
                          child: Container(
                            height: state.widget.actionsList.length * 50 + 6,
                            decoration: BoxDecoration(
                              color: Colors.white,
                              borderRadius: Artist().defaultBorderRadius,
                              border: Border.all(
                                color: const Color.fromARGB(255, 218, 218, 218),
                                width: 1,
                              ),
                            ),
                            child: ListView.separated(
                              separatorBuilder: (context, index) {
                                return Padding(
                                  padding: const EdgeInsets.symmetric(vertical: 6.0),
                                  child: Container(
                                    width: double.infinity,
                                    color: Colors.grey,
                                    height: 1,
                                  )
                                );
                              },
                              itemCount: state.widget.actionsList.length,
                              itemBuilder: (context, index) {
                                CustomAction action = state.widget.actionsList[index];

                                return SizedBox(
                                  height: 50,
                                  child: InkWell(
                                    onTap: () {
                                      action.callback();
                                      state.portalController.hide();
                                    },
                                    child: action.child,
                                  )
                                );
                              }
                            ),
                          )
                        )
                        // for (CustomAction action in state.widget.actionsList)
                        // action.child,
                      ]
                    ),
                  )
                );
              },
            ),
          ),
        );
      }
    );
  }
}
