
import 'package:flutter/material.dart';

import 'view.dart';

class OwlnextButton extends StatefulWidget {
  /// Owlnext Button
  /// Based on Material design and uses [ElevatedButton]
  /// 
  /// Can be used with [ButtonType] type, to determine default behavior
  /// Can be overrided with the props [customType], with a custom [CustomButtonType] to have specific theme
  const OwlnextButton({
    super.key,
    required this.callback,
    this.type = ButtonType.classic,
    this.isAsync = false,
    this.enabled = true,
    this.customType,
    required this.child,
    this.customWidth,
  });

  /// Callback on button click
  /// Must be async
  final Function callback;
  /// Button type
  /// Determines the background and text color
  /// Can be override with [customType]
  final ButtonType type;
  /// Determines the onClick behavior
  /// If true, a loader will be displayed until the Future returns
  final bool isAsync;
  /// Is enabled
  final bool enabled;
  /// If a [CustomButtonType] is fed, change the Elevated Button theme
  final CustomButtonType? customType;
  /// Widget to display inside the button
  final Widget child;
  /// Custom [double] width for the button
  final double? customWidth;

  @override
  OwlnextButtonState createState() {
    return OwlnextButtonState();
  }
}

class OwlnextButtonState extends State<OwlnextButton> with ViewMixin {
  bool isProcessing = false;

  Future<void> makeCallback() async {
    if (widget.isAsync == true) {
      setState(() {
        isProcessing = true;
      });

      await widget.callback();

      if (mounted) {
        setState(() {
          isProcessing = false;
        });
      }
    } else {
      widget.callback();
    }
  }
  
  @override
  Widget build(BuildContext context) {
    return render(context, this);
  }
}

enum ButtonType {
  classic,
  delete,
  save;
}

class CustomButtonType {
  final Color backgroundColor;

  CustomButtonType({
    required this.backgroundColor,
  });
}