

import 'package:flutter/material.dart';
import 'view.dart';

class OwlnextCheckbox extends StatefulWidget {
  /// Owlnext form input
  /// Based on Material design and uses [FormField]
  /// Used to automate all radio behaviors ine one place
  /// 
  /// Create a custom [FormField] in order to have form validation
  /// Refers to the props to see how to use
  const OwlnextCheckbox({
    super.key,
    required this.options,
    required this.selectedOptions,
    required this.handleChangeSelected,
    required this.label,
    this.infoCallback,
    this.isMandatory = false,
    this.enabled = true,
    this.disabledMessage
  });

  /// Options list to display and select from
  final List<CustomCheckboxOption> options;
  /// Current selected value
  final List<String> selectedOptions;
  /// Callback on option click
  /// Return the clicked option
  final void Function(CustomCheckboxOption) handleChangeSelected;
  /// Radio label
  final String label;
  /// Callback triggered when the '?' right of the label
  /// Doesn't display if null
  final Function? infoCallback;
  /// Determines the validator behavior
  final bool isMandatory;
  /// Determines if the radio is enabled
  final bool enabled;
  /// Message to show if the radio is disabled
  final String? disabledMessage;

  @override
  OwlnextCheckboxState createState() {
    return OwlnextCheckboxState();
  }
}

class OwlnextCheckboxState extends State<OwlnextCheckbox> with ViewMixin {
  @override
  void initState() {
    super.initState();
  }

  @override
  void dispose() {
    super.dispose();
  }
  
  @override
  Widget build(BuildContext context) {
    return render(context, this);
  }
}

class CustomCheckboxOption {
  Widget widget;
  String value;

  CustomCheckboxOption({
    required this.widget,
    required this.value,
  });
}