import 'package:desktop_drop/desktop_drop.dart';
import 'package:dotted_border/dotted_border.dart';
import 'package:file_picker/file_picker.dart';
import 'package:flutter/material.dart';
import 'package:nirrti/src/custom_components/custom_dropzone/document_icon_resolver/view_model.dart';
import 'package:nirrti/src/custom_components/custom_dropzone/view_model.dart';
import 'package:nirrti/src/styles/artist.dart';

mixin ViewMixin {
  render(BuildContext context, OwlnextDropzoneState state) {
    return Container(
      margin: const EdgeInsets.all(2),
      child: MouseRegion(
        cursor: SystemMouseCursors.click,
        child: GestureDetector(
          onTap: state.handlePickFile,
          child: DropTarget(
            onDragDone: (detail) async {
              state.dropFiles(detail.files);
            },
            onDragEntered: (detail) {
              state.handleChangeIsDragging(true);
            },
            onDragExited: (detail) {
              state.handleChangeIsDragging(false);
            },
            child: Column(
              children: [
                if (state.widget.size == DropZoneSize.normal)
                generateNormalDropZone(context, state),
                if (state.widget.size == DropZoneSize.small)
                generateSmallDropZone(context, state),
              ]
            ),
            
            
          )
        ),
      ),
    );
  }

  Widget generateSmallDropZone(BuildContext context, OwlnextDropzoneState state) {
    return DottedBorder(
      radius: const Radius.circular(4),
      borderType: BorderType.RRect,
      color: Colors.black.withOpacity(0.6),
      strokeWidth: 1,
      dashPattern: const [10, 2],
      child: Container(
        height: 70,
        width: 400,
        padding: Artist().defaultPadding,
        color: state.isDragging == true ? Theme.of(context).colorScheme.tertiary.withOpacity(0.3) : Colors.white,
        child: state.currentFilesToUpload.isEmpty ? Align(
          child: state.isDragging == true ? 
            const Icon(
              Icons.file_upload,
              color: Colors.white,
              size: 50,
            )
          : Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              Icon(
                Icons.file_upload,
                color: Colors.black.withOpacity(0.6),
                size: 35,
              ),
              Expanded(
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: [
                    Text(
                      'Importer un document ou une photo',
                      style: Theme.of(context).textTheme.bodyMedium?.copyWith(
                        color: Colors.black.withOpacity(0.6),
                        fontWeight: FontWeight.bold,
                      )
                    ),
                    Text(
                      "(Cliquez ici ou déposez des documents)",
                      style: Theme.of(context).textTheme.bodySmall?.copyWith(
                        color: Colors.black.withOpacity(0.6),
                      )
                    ),
                  ],
                ),
              )
            ],
          )
        ) :
        Row(
          children: [
            const SizedBox(
              width: 30,
              child: Icon(
                Icons.file_present,
                color: Colors.black,
                size: 35,
              )
            ),
            Expanded(
              child: Text(
                state.currentFilesToUpload[0].name,
                textAlign: TextAlign.center,
                style: Theme.of(context).textTheme.bodySmall?.copyWith(
                  
                )
              )
            ),
            SizedBox(
              width: 30,
              child: InkWell(
                onTap: () {
                  state.handleDeleteFileFromStack(state.currentFilesToUpload[0]);
                },
                child: Icon(
                  Icons.delete,
                  color: Theme.of(context).colorScheme.onError,
                  size: 20,
                )
              )
            )
          ]
        )
      )
    );
  }

  Widget generateNormalDropZone(BuildContext context, OwlnextDropzoneState state) {
    return DottedBorder(
      radius: const Radius.circular(4),
      borderType: BorderType.RRect,
      color: Colors.black.withOpacity(0.6),
      strokeWidth: 1,
      dashPattern: const [10, 2],
      child: state.widget.isMultiUpload == false ? Container(
        height: 105,
        width: double.infinity,
        padding: Artist().defaultPadding,
        color: state.isDragging == true ? Theme.of(context).colorScheme.tertiary.withOpacity(0.3) : Colors.white,
        child: state.currentFilesToUpload.isEmpty ? Align(
          child: generateText(context, 'Importer un document ou une photo', state)
        ) :
        Container(
          width: 120,
          decoration: BoxDecoration(
            borderRadius: Artist().defaultBorderRadius,
            color: const Color.fromARGB(255, 243, 243, 243)
          ),
          child: Stack(
            children: [
              Column(
                children: [
                  const Expanded(
                    child: Align(
                      child: Icon(
                        Icons.file_present,
                        color: Colors.black,
                        size: 35,
                      )
                    )
                  ),
                  SizedBox(
                    height: 40,
                    child: Align(
                      child: Text(
                        state.currentFilesToUpload[0].name,
                        textAlign: TextAlign.center,
                        style: Theme.of(context).textTheme.bodySmall?.copyWith(
                          
                        )
                      )
                    ),
                  ),
                ],
              ),
              Positioned(
                top: 5,
                right: 5,
                child: InkWell(
                  onTap: () {
                    state.handleDeleteFileFromStack(state.currentFilesToUpload[0]);
                  },
                  child: Icon(
                    Icons.delete,
                    color: Theme.of(context).colorScheme.onError,
                    size: 20,
                  )
                )
              )
            ],
          )
        )
      ) : Container(
        color: state.isDragging == true ? Theme.of(context).colorScheme.tertiary.withOpacity(0.3) : Colors.transparent,
        padding: Artist().defaultPadding,
        child: Column(
          children: [
            Container(
              height: 105,
              width: double.infinity,
              padding: Artist().defaultPadding,
              child: Align(
                child: generateText(context, 'Importer des documents ou des photos', state),
              )
            ),
            if (state.currentFilesToUpload.isNotEmpty)
            SizedBox(
              width: double.infinity,
              child: Wrap(
                spacing: 12.0,
                runSpacing: 12.0,
                children: [
                  for (PlatformFile file in state.currentFilesToUpload)
                  Container(
                    decoration: BoxDecoration(
                      color: const Color.fromARGB(255, 241, 241, 241),
                      borderRadius: BorderRadius.circular(8)
                    ),
                    padding: const EdgeInsets.all(12),
                    width: 275,
                    height: 80,
                    child: Row(
                      children: [
                        CustomDocumentIconResolver(
                          extension: file.extension ?? 'pdf',
                        ),
                        const SizedBox(
                          width: 12
                        ),
                        Expanded(
                          child: Column(
                            mainAxisAlignment: MainAxisAlignment.center,
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: [
                              Text(
                                file.name,
                                style: Theme.of(context).textTheme.bodySmall?.copyWith(
                                  fontWeight: FontWeight.bold,
                                  overflow: TextOverflow.ellipsis
                                )
                              ),
                              const SizedBox(
                                height: 12
                              ),
                              Text(
                                '${(file.size / 1000).toStringAsFixed(2)} Ko',
                                style: Theme.of(context).textTheme.bodySmall?.copyWith(
                                  overflow: TextOverflow.ellipsis
                                )
                              ),
                            ],
                          ),
                        ),
                        const SizedBox(
                          width: 8
                        ),
                        Align(
                          child: Container(
                            height: 50,
                            width: 1,
                            color: Colors.grey.withOpacity(0.3)
                          ),
                        ),
                        const SizedBox(
                          width: 8
                        ),
                        Align(
                          child: InkWell(
                            onTap: () {
                              state.handleDeleteFileFromStack(file);
                            },
                            child: MouseRegion(
                              cursor: SystemMouseCursors.click,
                              child: SizedBox(
                                height: 50,
                                child: Icon(
                                  Icons.delete,
                                  color: Theme.of(context).colorScheme.onError,
                                ),
                              )
                            )
                          )
                        )
                      ],
                    ),
                  )
                ]
              ),
            )
          ]
        ),
      )
    );
  }

  Widget generateText(BuildContext context, String text, OwlnextDropzoneState state) {
    return state.isDragging == true ? 
      const Icon(
        Icons.file_upload,
        color: Colors.white,
        size: 50,
      )
    : Column(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: [
        Icon(
          Icons.file_upload,
          color: Colors.black.withOpacity(0.6),
          size: 40,
        ),
        Text(
          text,
          style: Theme.of(context).textTheme.bodyMedium?.copyWith(
            color: Colors.black.withOpacity(0.6),
            fontWeight: FontWeight.bold
          )
        ),
        Text(
          "(Cliquez ici ou déposez des documents)",
          style: Theme.of(context).textTheme.bodySmall?.copyWith(
            color: Colors.black.withOpacity(0.6),
          )
        ),
      ],
    );
  }
}
