

import 'package:cross_file/cross_file.dart';
import 'package:file_picker/file_picker.dart';
import 'package:flutter/material.dart';
import 'package:nirrti/src/utils/error/error_type.dart';
import 'package:nirrti/src/utils/file/file_utils.dart';
import 'view.dart';

class OwlnextDropzone extends StatefulWidget {
  const OwlnextDropzone({
    super.key,
    required this.fileUtils,
    this.allowedExtensions = const ['pdf', 'jpeg', 'png', 'jpg'],
    this.isMultiUpload = false,
    this.size = DropZoneSize.normal
  });

  /// [FileUtils] has to be instanciated to manage the files
  /// Contains the [FileUtils.filesList] and the [FileUtils.isMultiUpload] properties
  final FileUtils fileUtils;
  /// Extensions allowed to be uploaded
  final List<String> allowedExtensions;
  /// Dropzone handles multiupload
  final bool isMultiUpload;
  /// [DropZoneSize.small] or [DropZoneSize.normal] depending on the layout
  final DropZoneSize size;

  @override
  OwlnextDropzoneState createState() {
    return OwlnextDropzoneState();
  }
}

class OwlnextDropzoneState extends State<OwlnextDropzone> with ViewMixin {
  List<PlatformFile> currentFilesToUpload = [];
  bool isDragging = false;

  @override
  void initState() {
    super.initState();
    initFiles();
  }

  @override
  void dispose() {
    super.dispose();
  }

  void initFiles() {
    if (widget.fileUtils.filesList.isNotEmpty) {
      setState(() {
        currentFilesToUpload = widget.fileUtils.filesList;
      });
    }
  }

  void handleChangeIsDragging(bool value) {
    setState(() {
      isDragging = value;
    });
  }

  void handlePickFile() async {
    try {
      List<PlatformFile> files = await widget.fileUtils.handlePickFile(widget.allowedExtensions);
     
      if (mounted) {
        setState(() {
          currentFilesToUpload = files;
        });
      }
    } catch (e) {
      if (e is ErrorType) {
        rethrow;
      }
    }
  }

  void handleDeleteFileFromStack(PlatformFile file) {
    List<PlatformFile> files = widget.fileUtils.handleDeleteFileFromStack(file);

    setState(() {
      currentFilesToUpload = files;
    });
  }

  void dropFiles(List<XFile> filesList) async {
    try {
      List<PlatformFile> files = await widget.fileUtils.handleDropFiles(filesList, widget.allowedExtensions);

      setState(() {
        currentFilesToUpload = files;
      });
    } catch (e) {
      rethrow;
    }
  }

  @override
  Widget build(BuildContext context) {
    return render(context, this);
  }
}

enum DropZoneSize {
  small,
  normal,
}
