
import 'package:flutter/material.dart';
import 'package:nirrti/nirrti.dart';
import 'package:nirrti/src/extensions/string_extensions.dart';
import 'view_model.dart';

mixin ViewMixin {
  render(BuildContext context, OwlnextFormFieldGeneratorState state) {
    return Padding(
      padding: EdgeInsets.only(bottom: Artist().listSeparatorHeight),
      child: generateInput(context, state.widget.field, state),
    );
  }

  Widget generateInput(BuildContext context, FieldsDetails? item, OwlnextFormFieldGeneratorState state) {
    if(item == null) {
      return const SizedBox.shrink();
    }

    switch (item.displayType) {
      case FieldType.text: 
        return OwlnextInput(
          label: item.label.capitalize(),
          controller: state.text,
        );
      case FieldType.textarea: 
        return OwlnextInput(
          label: item.label.capitalize(),
          controller: state.text,
          minimumLines: 5,
        );
      case FieldType.phone: 
        return OwlnextInput(
          label: item.label.capitalize(),
          controller: state.text,
          type: TextInputSubType.numbers,
          charsLimit:10
        );
      case FieldType.date: 
        return OwlnextInput(
          label: item.label.capitalize(),
          controller: state.text,
          type: TextInputSubType.date,
        );
      case FieldType.number: 
        return OwlnextInput(
          label: item.label.capitalize(),
          controller: state.text,
          type: TextInputSubType.numbers,
        );
      case FieldType.amount:
        return OwlnextInput(
          label: item.label.capitalize(),
          controller: state.text,
          type: TextInputSubType.amount,
          hintText: state.widget.field?.hintText,
          suffixText: state.widget.field?.suffixText,
        );
      case FieldType.zipCode: 
        return OwlnextInput(
          label: item.label.capitalize(),
          controller: state.text,
          charsLimit: 5
        );
      case FieldType.select:
        return OwlnextSelect(
          label: item.label.capitalize(),
          value: state.selectedOption, 
          onChanged: (String value) {
            // ignore: invalid_use_of_protected_member
            state.setState(() {
              state.selectedOption = value;
            });
            state.populatePostFields(value);
          },
          items: [
            for(int i = 0; i < item.options!.length; i++)
            DropdownMenuItem(value: item.options![i]!.value, child: Text(item.options![i]!.label))
          ],
        );
      case FieldType.complex: 
        return Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            if(item.fields != null && item.fields!.isNotEmpty)
            for(int i = 0 ; i < item.fields!.length ; i++)
            OwlnextFormFieldGenerator(
              field: item.fields![i],
              arrayKey : state.widget.arrayKey,
              postFields : state.widget.postFields,
              arrayKeyChild: i,
              method: state.widget.method,
            ),
          ],
        );
      default:
        return const SizedBox();
    }
  }
}
