

import 'package:flutter/material.dart';
import 'package:nirrti/src/utils/formfield_generator/field_details.dart';
import 'package:nirrti/src/utils/formfield_generator/field_post_input.dart';
import 'view.dart';

class OwlnextFormFieldGenerator extends StatefulWidget {
  /// Owlnext [FormField] generator
  /// 
  /// Depending on the configuration, generate a list of fields based
  /// on Owlnext fields components [OwlnextSelect, OwlnextInput...]
  /// 
  /// Can be used as a standalone to quickly generate multiple forms
  const OwlnextFormFieldGenerator({
    super.key,
    required this.field,
    this.arrayKeyChild,
    required this.arrayKey,
    required this.postFields,
    this.method = FormMethod.post,
  });

  /// Field to render
  final FieldsDetails? field;
  final int arrayKey;
  final List<FieldsPostInput> postFields;
  final int? arrayKeyChild;
  final FormMethod method;

  @override
  OwlnextFormFieldGeneratorState createState() {
    return OwlnextFormFieldGeneratorState();
  }
}

class OwlnextFormFieldGeneratorState extends State<OwlnextFormFieldGenerator> with ViewMixin {
  TextEditingController text = TextEditingController();
  String? selectedOption;
  
  @override
  void initState() {
    text.addListener(handleChangeTextInput);
    if(widget.method == FormMethod.put){
      setState(() {
        if(widget.field?.displayType == FieldType.select){
          selectedOption = widget.field?.value;
        } else if (widget.field?.displayType == FieldType.complex){

        } else {
          text.text = widget.field!.value ?? '';
        }
      });
    }
   
    super.initState();
  }

  @override
  void dispose() {
    text.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return render(context, this);
  }

   void handleChangeTextInput() {
    populatePostFields(text.text); 
  }

  void populatePostFields(String value) {
    if(widget.arrayKeyChild != null){
      widget.postFields[widget.arrayKey].fields![widget.arrayKeyChild ?? 0 ]!.value = value;
    } else {
      widget.postFields[widget.arrayKey].value = value;
    }
  }
}

enum FormMethod {
  post,
  put
}
