
import 'package:flutter/material.dart';
import 'package:nirrti/nirrti.dart';
import 'package:nirrti/src/custom_components/custom_helper/view_model.dart';
import 'package:url_launcher/url_launcher.dart';


mixin ViewMixin {
  render(BuildContext context, OwlnextHelper state) {
    return Container(
      padding: Artist().defaultPadding,
      margin: const EdgeInsets.only(bottom: 12),
      decoration: BoxDecoration(
        color: state.type == TooltipType.classic ?
          Theme.of(context).colorScheme.tertiary.withOpacity(0.2)
          :
          state.type == TooltipType.warning ?
            Colors.orange.withOpacity(0.3)
            :
            state.type == TooltipType.error ?
              Theme.of(context).colorScheme.onError
            :
            Colors.white,
        borderRadius: BorderRadius.circular(8)
      ),
      child: Row(
        children: [
          if (state.hasIcon == true && state.type == TooltipType.classic)
          Icon(
            Icons.info,
            color: Theme.of(context).colorScheme.tertiary 
          ),
          if (state.hasIcon == true && state.type == TooltipType.warning)
          const Icon(
            Icons.warning,
            color: Color.fromARGB(255, 179, 114, 16)
          ), 
          if (state.hasIcon == true && state.type == TooltipType.error)
          const Icon(
            Icons.error,
            color: Colors.white
          ), 
          const SizedBox(
            width: 12
          ),
          Expanded(
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Text(
                  state.message,
                  overflow: TextOverflow.clip,
                  style: TextStyle(
                    color: state.type == TooltipType.classic ?
                      Theme.of(context).colorScheme.tertiary 
                      :
                      state.type == TooltipType.warning ?
                        const Color.fromARGB(255, 179, 114, 16)
                        : 
                        state.type == TooltipType.error ?
                          Colors.white
                        :
                        Colors.white 
                    ,
                    fontSize: 12
                  ),
                ),
                if (state.link != '')
                InkWell(
                  onTap: () {
                    launchUrl(Uri.parse(state.link));
                  },
                  child: Text(
                    state.link,
                    style: TextStyle(
                      color: state.type == TooltipType.classic ?
                        Theme.of(context).colorScheme.tertiary 
                        :
                        state.type == TooltipType.warning ?
                          const Color.fromARGB(255, 179, 114, 16)
                          : 
                          state.type == TooltipType.error ?
                            Colors.white
                          :
                          Colors.white 
                      ,
                      fontSize: 12
                    ),
                  ),
                ),
              ],
            ),
          ),
        ],
      )
    );
  }
}
