import 'dart:developer';
import 'package:ensure_visible_when_focused/ensure_visible_when_focused.dart';
import 'package:material_design_icons_flutter/material_design_icons_flutter.dart';
import 'package:nirrti/nirrti.dart';
import 'package:nirrti/src/extensions/string_extensions.dart';
import 'package:nirrti/src/formatters/date_formatter.dart';
import 'package:nirrti/src/formatters/hours_formatter.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:intl/intl.dart';
import './view_model.dart';

mixin ViewMixin {
  render(BuildContext context, OwlnextInputState state) {
    if (state.widget.axis == Axis.vertical) {
      return Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          if (state.widget.label != null && Artist().inputDecoration.floatingLabelBehavior != FloatingLabelBehavior.auto)
          SizedBox(
            child: Row(
              children: [
                if (state.widget.label != null)
                Expanded(
                  child: Text(
                    "${state.widget.label ?? ''} ${state.widget.isMandatory == true ? "* " : ""}",
                    textAlign: TextAlign.left,
                    style: state.getLabelStyle
                  ),
                ),
                if (state.widget.infoCallback != null) 
                InkWell(
                  child: MouseRegion(
                    cursor: SystemMouseCursors.click,
                    child: GestureDetector(
                      onTap: () {
                        state.widget.infoCallback!();
                      },
                      child: Container(
                        width: 17,
                        height: 17,
                        margin: const EdgeInsets.only(left: 12),
                        padding: const EdgeInsets.all(2),
                        decoration: BoxDecoration(
                          border: Border.all(
                            color: Theme.of(context).colorScheme.primary,
                          ),
                          borderRadius: BorderRadius.circular(100)
                        ),
                        child: Icon(
                          Icons.question_mark,
                          color: Theme.of(context).colorScheme.primary,
                          size: 10
                        )
                      )
                    )
                  )
                )
              ]
            ),
          ),
          if (state.widget.label != null && Artist().inputDecoration.floatingLabelBehavior != FloatingLabelBehavior.auto)
          SizedBox(
            height: Artist().labelBottomSpace,
          ),
          Row(
            children: [
              Flexible(
                child: state.widget.isInsideList == true ? 
                  ConstrainedBox(
                    constraints: BoxConstraints(
                      minWidth: 250,
                      maxWidth: Artist().maxInputWidth,
                    ),
                    child: EnsureVisibleWhenFocused(
                      focusNode: state.focusNode,
                      child: generateForm(context, state.focusNode, state),
                    ),
                  ) : 
                  ConstrainedBox(
                    constraints: BoxConstraints(
                      minWidth: 250,
                      maxWidth: Artist().maxInputWidth,
                    ),
                    child: generateForm(context, state.focusNode, state)
                  )
              ),
              if (state.widget.hasCopyToClipboard)
              Row(
                children: [
                  const SizedBox(width: 12),
                  InkWell(
                    onTap: state.handleCopyToClipBoard,
                    child: Icon(
                      MdiIcons.contentCopy,
                      color: Theme.of(context).colorScheme.primary,
                    )
                  ),
                ],
              )
            ],
          ),
        ]
      );
    } else if (state.widget.axis == Axis.horizontal) {
      return Row(
        crossAxisAlignment: state.widget.minimumLines! > 1 ? CrossAxisAlignment.start : CrossAxisAlignment.center,
        children: [
          if (state.widget.label != null && Artist().inputDecoration.floatingLabelBehavior != FloatingLabelBehavior.auto)
          SizedBox(
            child: Row(
              children: [
                if (state.widget.label != null)
                SizedBox(
                  width: Artist().horizontalLabelWidth,
                  child: Text(
                    "${state.widget.label ?? ''} ${state.widget.isMandatory == true ? "* " : ""}",
                    textAlign: TextAlign.left,
                    style: state.getLabelStyle
                  ),
                ),
                if (state.widget.infoCallback != null) 
                InkWell(
                  child: MouseRegion(
                    cursor: SystemMouseCursors.click,
                    child: GestureDetector(
                      onTap: () {
                        state.widget.infoCallback!();
                      },
                      child: Container(
                        width: 17,
                        height: 17,
                        margin: const EdgeInsets.only(left: 12),
                        padding: const EdgeInsets.all(2),
                        decoration: BoxDecoration(
                          border: Border.all(
                            color: Theme.of(context).colorScheme.primary,
                          ),
                          borderRadius: BorderRadius.circular(100)
                        ),
                        child: Icon(
                          Icons.question_mark,
                          color: Theme.of(context).colorScheme.primary,
                          size: 10
                        )
                      )
                    )
                  )
                )
              ]
            ),
          ),
          if (state.widget.label != null && Artist().inputDecoration.floatingLabelBehavior != FloatingLabelBehavior.auto)
          SizedBox(
            width: Artist().labelBottomSpace,
          ),
          Flexible(
            child: ConstrainedBox(
              constraints: BoxConstraints(
                minWidth: 250,
                maxWidth: Artist().maxInputWidth,
              ),
              child: state.widget.isInsideList == true ? 
                EnsureVisibleWhenFocused(
                  focusNode: state.focusNode,
                  child: generateForm(context, state.focusNode, state),
                ) : 
                generateForm(context, state.focusNode, state),
            )
          ),
          if (state.widget.hasCopyToClipboard)
          Row(
            children: [
              const SizedBox(width: 12),
              InkWell(
                onTap: state.handleCopyToClipBoard,
                child: Icon(
                  MdiIcons.contentCopy,
                  color: Theme.of(context).colorScheme.primary,
                )
              ),
            ],
          )
        ]
      );
    }
    
  }

  Widget generateForm(BuildContext context, FocusNode currentFocus, OwlnextInputState state) {
    return TextFormField(
      textInputAction: state.widget.textInputAction,
      onFieldSubmitted: (String val) {
        if (state.widget.textInputAction != null){
          currentFocus.unfocus();
          // FocusScope.of(context).requestFocus(nextFocus);
        } else {
          state.widget.onSubmitted!(val);
        }
      },
      focusNode: currentFocus,
      onEditingComplete: () {},
      enabled: state.widget.enabled,
      minLines: state.widget.minimumLines,
      maxLines: state.widget.minimumLines,
      keyboardType: [TextInputSubType.date, TextInputSubType.shortDate].contains(state.widget.type) ? TextInputType.datetime
        : [TextInputSubType.hours, TextInputSubType.numbers].contains(state.widget.type) ? TextInputType.number
        : null,
      inputFormatters: formatter(state),
      decoration: Artist().inputDecoration.copyWith(
        isDense: true,
        hintText: generateHintText(state),
        errorText: state.widget.errorText != '' ? state.widget.errorText : null,
        fillColor: state.widget.enabled == true ? Colors.white : Artist().disabledFilledInputColor,
        suffixIcon: state.getSuffixIcon,
        prefixIcon: state.widget.prefixIcon,
        floatingLabelBehavior: FloatingLabelBehavior.auto,
        labelText: Artist().inputDecoration.floatingLabelBehavior == FloatingLabelBehavior.auto ?
          state.widget.label : null,
      ),
      style: Artist().innerInputTextStyle,
      obscureText: state.widget.type == TextInputSubType.password,
      controller: state.widget.controller,
      initialValue: state.widget.text,
      onTap: () {
        // FocusScope.of(context).requestFocus(FocusNode());
      },
      onChanged: (value) {
        // FocusScope.of(context).requestFocus(FocusNode());
        
        if (state.widget.onChanged != null) {
          state.widget.onChanged!(value);
        }
      },
      validator: (String? value) => validator(state, currentFocus, value),
    );
  }

  String? validator(OwlnextInputState state, FocusNode currentFocus, String? value) {
    if (state.widget.isMandatory == true) {
      try {
        if (value!.isEmpty) {
          currentFocus.requestFocus();
          return state.widget.errorText != '' ? state.widget.errorText : 'Merci de remplir ce champ';
        }
      
        if (state.widget.type == TextInputSubType.date) {
          try {
            DateFormat('dd/MM/yyyy').parse(value);
      
            if (state.widget.minimumDate != null && DateFormat('dd/MM/yyyy').parse(value).isBefore(state.widget.minimumDate!.subtract(const Duration(days: 1))) == true) {
              currentFocus.requestFocus();
              return 'Cette date doit être supérieure au ${DateFormat.yMMMMEEEEd('fr').format(state.widget.minimumDate!).capitalize()}';
            }

            if (state.widget.maximumDate != null && DateFormat('dd/MM/yyyy').parse(value).isAfter(state.widget.maximumDate!) == true) {
              currentFocus.requestFocus();
              return 'Cette date doit être inférieure au ${DateFormat.yMMMMEEEEd('fr').format(state.widget.maximumDate!).capitalize()}';
            }
      
            return null;
          } catch (e) {
            currentFocus.requestFocus();
            return "Cette date n'est pas valide";
          }
        }
      
        if (state.widget.type == TextInputSubType.email) {
          RegExp test = RegExp(r'^[\w-\.+]+@([\w-]+\.)+[\w-]{2,4}$');
      
          if (test.hasMatch(value)) {
            return null;
          } else {
            currentFocus.requestFocus();
            return "Merci d'entrer un email valide";
          }
      
        }
      } catch (e) {
        currentFocus.requestFocus();
        return 'Une erreur est survenue au formattage de ce champ';
      }
      
      return null;
    } else {
      return null;
    }
  }

  List<TextInputFormatter>? formatter(OwlnextInputState state) {
    List<TextInputFormatter> arr = [];

    if (state.widget.type == TextInputSubType.amount) {
      arr = [
        FilteringTextInputFormatter.allow(RegExp(r"[0-9,.]")),
        TextInputFormatter.withFunction((oldValue, newValue) {
          try {
            String text = newValue.text;

            if (text.contains(',') == true) {
              text = text.replaceAll(',', '.');
            }

            if (text.isNotEmpty && state.widget.amountMaxLimit != null && double.parse(text) > state.widget.amountMaxLimit!) {
              return oldValue;
            }

            int limit = 2;

            if (state.widget.decimalsMaxLimit != null) {
              limit = state.widget.decimalsMaxLimit!;
            }

            List<String> splitted = text.split('.');

            if (splitted.length >= 2 && splitted[1].length > limit) {
              return oldValue;
            }

            if (text.isNotEmpty) double.parse(text);

            if (text.contains('.')) {
              TextEditingValue finalController = newValue.copyWith(
                text: newValue.text.replaceAll('.', ',')
              );
              
              return finalController;
            }
            return newValue;
          } catch (e) {
            log(e.toString());
          }
          return oldValue;
        }),
      ];
    }

    if (state.widget.type == TextInputSubType.numbers) {
      arr = [
        FilteringTextInputFormatter.allow(RegExp(r"[0-9]")),
        TextInputFormatter.withFunction((oldValue, newValue) {
          try {
            final text = newValue.text;

            if (text.isNotEmpty) {
              int.parse(text);
            } else {
              return newValue;
            }

            if (state.widget.numberMaxLimit != null && int.parse(newValue.text) > int.parse(state.widget.numberMaxLimit.toString())) {
              return oldValue;
            }

            if (state.widget.numberMinLimit != null && int.parse(newValue.text) < int.parse(state.widget.numberMinLimit.toString())) {
              return oldValue;
            }

            return newValue;
          } catch (e) {
            log(e.toString());
          }
          return oldValue;
        }),
      ];
    }

    if (state.widget.type == TextInputSubType.date) {
      List<TextInputFormatter> temp = [];

      try {
        temp = [
          FilteringTextInputFormatter.allow(RegExp(r"[0-9/]")),
          LengthLimitingTextInputFormatter(10),
          DateFormatter(),
        ];
      } catch (e) {
        temp = [
          FilteringTextInputFormatter.allow(RegExp(r"[0-9/]")),
          LengthLimitingTextInputFormatter(10)
        ];
      }

      arr = temp;
    }

    if (state.widget.type == TextInputSubType.shortDate) {
      List<TextInputFormatter> temp = [];

      try {
        temp = [
          FilteringTextInputFormatter.allow(RegExp(r"[0-9/]")),
          LengthLimitingTextInputFormatter(5),
          ShortDateFormatter(),
        ];
      } catch (e) {
        temp = [
          FilteringTextInputFormatter.allow(RegExp(r"[0-9/]")),
          LengthLimitingTextInputFormatter(5)
        ];
      }

      arr = temp;
    }

    if (state.widget.type == TextInputSubType.hours) {
      List<TextInputFormatter> temp = [];

      try {
        temp = [
          FilteringTextInputFormatter.allow(RegExp(r"[0-9:]")),
          LengthLimitingTextInputFormatter(10),
          HoursFormatter(),
        ];
      } catch (e) {
        temp = [
          FilteringTextInputFormatter.allow(RegExp(r"[0-9:]")),
          LengthLimitingTextInputFormatter(10)
        ];
      }

      arr = temp;
    }

    if (state.widget.charsLimit != null) {
      arr.add(
        LengthLimitingTextInputFormatter(state.widget.charsLimit)
      );
    }

    if (state.widget.customRegex != null) {
      arr = [
        FilteringTextInputFormatter.allow(state.widget.customRegex!)
      ];
    }

    return arr;
  }

  String generateHintText(OwlnextInputState state) {
    if (state.widget.hintText != '') {
      return state.widget.hintText ?? '';
    } else if (state.widget.type == TextInputSubType.date) {
      return 'JJ/MM/AAAA';
    } else if (state.widget.type == TextInputSubType.shortDate) {
      return 'JJ/MM';
    } else if (state.widget.type == TextInputSubType.hours) {
      return 'hh:mm';
    }

    return state.widget.hintText ?? '';
  }
}
