
import 'package:flutter/material.dart';

import 'view.dart';

/// Show a custom loader on the screen
/// Used to have the same behavior everywhere on the app
/// Uses the theme [primaryColor]
class OwlnextLoading extends StatefulWidget {
  const OwlnextLoading({
    this.alignment,
    super.key
  });

  /// Setup the alignement
  /// [Alignment.center] by default
  final Alignment? alignment;

  @override
  OwlnextLoadingState createState() {
    return OwlnextLoadingState();
  }
}

class OwlnextLoadingState extends State<OwlnextLoading> with TickerProviderStateMixin, ViewMixin {
  late AnimationController animationController;

  @override
  void initState() {
    animationController = AnimationController(
      vsync: this,
      duration: const Duration(seconds: 3),
    )..addListener(() {
      setState(() {});
    });
    animationController.repeat(reverse: true);
    super.initState();
  }

  @override
  void dispose() {
    animationController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return render(context, this);
  }
}