
import 'package:flutter/material.dart';
import 'view.dart';

class OwlnextSelect extends StatefulWidget {
  /// Owlnext form input
  /// Based on Material design and uses [DropdownButtonFormField]
  /// Used to automate all selects behaviors ine one place
  /// 
  /// Can be customized through a variety of properties, like [items] or [optionsRoute]
  /// depending on the options getter methods
  const OwlnextSelect({
    super.key,
    required this.value,
    required this.onChanged,
    this.items,
    this.label,
    this.enabled = true,
    this.isMandatory = false,
    this.addButtonCallback,
    this.infoCallback,
    this.optionsRoute,
    this.axis = Axis.vertical,
  }) :
    assert((items != null && optionsRoute == null) || (items == null && optionsRoute != null));

  /// Current selected value
  final String? value;
  /// List of items to display
  /// Useful if you want to write the options without an API Response
  final List<DropdownMenuItem<String>>? items;
  /// Callback when a value is selected
  final void Function(String newValue) onChanged;
  /// Select input label
  final String? label;
  /// Is enabled or not
  final bool? enabled;
  /// Is mandatory or not, determines the validator behavior
  final bool? isMandatory;
  /// If not null, adds a "+" button to the Select
  /// This callback is called when the button is pressed
  final Function? addButtonCallback;
  /// Called when clicking on the information icon, right to the label
  /// Can be anything (openPanel, openPopup...)
  final Function? infoCallback;
  /// Url to call in order to have the options
  /// Useful if you want your options list to be async
  final Future<List<DropdownMenuItem<String>>> Function()? optionsRoute;
  /// Set the axis for the label -> input
  final Axis? axis;

  @override
  OwlnextSelectState createState() {
    return OwlnextSelectState();
  }
}

class OwlnextSelectState extends State<OwlnextSelect> with ViewMixin {
  List<DropdownMenuItem<String>> currentItems = [];
  String? currentValue;
  UniqueKey renderKey = UniqueKey();

  @override
  void initState() {
    initOptions();
    super.initState();
  }

  /// Initializes the select options
  /// depending on the selected items method
  void initOptions() async {
    setState(() {
      currentValue = widget.value?.isEmpty == true ? null : widget.value;
    });

    if (widget.items != null) {
      setState(() {
        currentItems = widget.items ?? [];
        renderKey = UniqueKey();
      });
    } else if (widget.optionsRoute != null) {
      List<DropdownMenuItem<String>> result = await widget.optionsRoute!();

      setState(() {
        currentItems = result;
        renderKey = UniqueKey();
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return render(context, this);
  }
}