

import 'package:intl/intl.dart';

extension StringExtension on String {
  String capitalize() {
    try {
      return "${this[0].toUpperCase()}${substring(1)}";
    } catch (e) {
      return '';
    }
  }

  double parseInputToDouble() {
    return double.parse(replaceAll(' ', '').replaceAll(',', '.'));
  }

  double? tryParseInputToDouble() {
    return double.tryParse(replaceAll(' ', '').replaceAll(',', '.'));
  }

  DateTime formatISOToDateTime () {
    String isoDate = substring(0, length - 6).replaceAll('T', ' ');
    DateTime newDateTime = DateTime.parse(isoDate);
    
    return newDateTime;
  }

  DateTime formatInputToDateTime({ String? specificFormat }) {
    return DateFormat(specificFormat ?? 'dd/MM/yyyy').parse(this);
  }

  DateTime formatShortInputToDateTime({ String? specificFormat }) {
    return DateFormat(specificFormat ?? 'dd/MM').parse(this);
  }

  int getDayFromShortDate() {
    return int.parse(split('/')[0]);
  }

  int getMonthFromShortDate() {
    return int.parse(split('/')[1]);
  }

  String snakeToCamel() {
    List<String> splitted = split('_');
    String output = '';

    for (String item in splitted) {
      String value = item;

      if (item != splitted.first) {
        value = item.capitalize();
      }

      output += value;
    }

    return output;
  }

  String formatPhone() {
    var allNumbers = RegExp(r'(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})');
    String finalString = replaceAllMapped(allNumbers, (Match m) {
      return '${m[1]} ${m[2]} ${m[3]} ${m[4]} ${m[5]}';
    });

    return finalString;
  }
}