import 'dart:developer';
import 'package:cross_file/cross_file.dart';
import 'package:file_picker/file_picker.dart';
import 'package:url_launcher/url_launcher.dart';
  
class FileUtils{
  List<PlatformFile> filesList = [];
  bool isMultiUpload;

  FileUtils({
    this.isMultiUpload = false,
  });

  Future<List<PlatformFile>> handlePickFile(List<String> allowedExtensions) async {    
    try {
      var currentFiles = (await FilePicker.platform.pickFiles(
        type: FileType.custom,
        allowedExtensions: allowedExtensions,
        allowMultiple: true,
      ))
      ?.files;

      for (PlatformFile currentFile in currentFiles!) {
        checkFileError(currentFile, allowedExtensions);
        
        if (isMultiUpload) {
          filesList.add(currentFile);
        } else {
          filesList = [currentFile];
        }        
      }

      return filesList;
    } catch (e) {
      log(e.toString());
      rethrow;
    }
  }

  Future<List<PlatformFile>> handleDropFiles(List<XFile> droppedFiles, List<String> allowedExtensions) async {
    try {
      for (XFile file in droppedFiles) {
        PlatformFile currentFile = PlatformFile(
          name: file.name,
          size: await file.length(),
          bytes: await file.readAsBytes()
        );

        checkFileError(currentFile, allowedExtensions);

        if (isMultiUpload) {
          filesList.add(currentFile);
        } else {
          filesList = [currentFile];
        }      
      }

      return filesList;
    } catch (e) {
      rethrow;
    }
  }

  List<PlatformFile> handleDeleteFileFromStack(PlatformFile file) {
    filesList.removeWhere((element) => element == file);

    return filesList;
  }

  void checkFileError(PlatformFile file, List<String> allowedExtensions) {
    if (file.size > 10000000) {
      filesList.clear();

      throw(FileError.fileTooBig);
    } else if (allowedExtensions.contains(file.extension) == false) {
      filesList.clear();

      throw(FileError.extensionError);
    }
  }

  Future<void> launchUrlFromUri(Uri uri) async {
    if (!await launchUrl(uri, mode: LaunchMode.externalNonBrowserApplication)) {
      throw 'Could not launch $uri';
    }
  }
}

enum FileError {
  fileTooBig,
  extensionError,
  genericError,
}