import 'dart:developer';

import 'package:uuid/uuid.dart';
import 'generic_list.dart';

class FieldsDetails {
    String? id;
    late String label;
    late FieldType displayType;
    late String valueType;
    late List<FieldsDetails?>? fields;
    late List<GenericList?>? options;
    late bool required;
    late String? value;
    late String? hintText;
    String? suffixText;
    List<TargetResolver>? trigger;
    late bool hasCopy;

    FieldsDetails({
      this.id,
      required this.label,
      required this.displayType,
      required this.valueType,
      this.fields,
      this.options,
      required this.required,
      this.value,
      this.hintText,
      this.suffixText,
      this.trigger,
      this.hasCopy = false,
    });

    FieldsDetails.fromJson(dynamic input) {
      try {
        label = input['label'].toString();
        valueType = input['valueType'];
        fields = null;
        // fields =  input['fields'] != null ? [...input['fields'].map((e) => FieldsDetails.fromJson(e))] : null;
        required = input['required'];
        value = input['value'];
        hintText = input['hintText'];
        suffixText = input['suffixText'];


        for (int i = 0; i < FieldType.values.length; i ++) {
          if (FieldType.values[i].name == input['displayType']) {
            displayType = FieldType.values[i];
          }
        }

        if (input['trigger'] != null) {
          trigger = [...input['trigger'].map((e) => TargetResolver.fromJson(e))];
        }

        id = input['id'] ?? const Uuid().v1();
        hasCopy = input['hasCopy'] ?? false;
        options =  input['options'] != null ? [...input['options'].map((e) => GenericList.fromJson(e))] : null;
      } catch(e, stacktrace) {
        log(e.toString());
        log('Function: FieldsDetails.fromJson()');
        log(stacktrace.toString());
      }
    }


    Map<String, dynamic> toJson() {
      try {
        final Map<String, dynamic> data = <String, dynamic>{};

        for (int i = 0; i < FieldType.values.length; i ++) {
          if (FieldType.values[i] == displayType) {
            data['displayType'] = FieldType.values[i].name;
          }
        }

        data['id'] = id;
        data['label'] = label;
        data['valueType'] = valueType;
        data['fields'] = fields != null ? [...fields!.map((e) => e!.toJson())] : null;
        data['options'] = options != null ? [...options!.map((e) => e!.toJson())] : null;
        data['required'] = required;
        data['value'] = value;
        data['hintText'] = hintText;
        data['suffixText'] = suffixText;
        data['trigger'] = trigger != null ? [...trigger!.map((e) => e.toJson())] : null;
        data['hasCopy'] = hasCopy;

        return data;
      } catch (e) {
        log("----------------------------");
        log(e.toString());
      }

      return {};
    }
}

class TargetResolver {
  late String targettedField;
  late String targettedValue;

  TargetResolver({
    required this.targettedField,
    required this.targettedValue,
  });

  TargetResolver.fromJson(dynamic input) {
    targettedField = input['targettedField'];
    targettedValue = input['targettedValue'].toString();
  }

  Map<String, dynamic> toJson() {
    Map<String, dynamic> data = {};

    data['targettedField'] = targettedField;
    data['targettedValue'] = targettedValue;

    return data;
  }
}

enum FieldType {
  text,
  textarea,
  phone,
  date,
  shortDate,
  number,
  zipCode,
  select,
  complex,
  amount,
  radio,
  checkbox,
  repeatableBlock
}